<?php
/*
Plugin Name: Peter's Custom Anti-Spam
Plugin URI: http://www.theblog.ca/anti-spam
Description: Stop a lot of spambots from polluting your site by making visitors identify a random word displayed as an image before commenting. You can customize the pool of words to display. Core of the Version 2.0 rewrite by <a href="http://WillMurray.name/">Will Murray</a>.
Author: Peter Keung
Version: 3.0.2
Author URI: http://www.theblog.ca/
Change Log:
2008-03-13  Version 3.0.2  Minor code tweak to be compatible with WordPress 2.5.
2008-02-10  Version 3.0.1  New option to disable random stretching of the wav files. Made the operation to stretch the wav files silent to the server(thanks Angelo!). Blocked indexing of audio file links by search engines (thanks Lucky!).
2008-01-20  Version 3.0.0  Added registration form protection and a corresponding e-mail blacklist. To enable registration form protection, set $cas_reg_protection equal to "true". Also tweaked the audio file generation with wav files to be more robust against spambots.
2007-12-02  Version 2.9.8  Added support for spaces and hyphens in the anti-spam words.
2007-11-20  Version 2.9.7  Minor code tweak to the image and audio generation code so that full page generation doesn't occur unnecessarily (thanks Steve!).
2007-11-11  Version 2.9.6  Made a few minor efficiency and display tweaks, and switched up the fonts.
2007-10-13  Version 2.9.5  Tweaked a database command to be more compatible, allowed a site address override (over the WordPress site URL value), and removed some redundant code.
2007-09-30  Version 2.9.4  Fixed a bug that was blocking trackbacks and pingbacks for the past few versions (a stupid oversight on my part in the order of some code).
2007-09-26  Version 2.9.3  Fixed minor CSS detail, allowed for different folder names for the plugin, and renamed mp3 class to avoid the possibility of clashes with other class names.
2007-09-10  Version 2.9.2  Fixed a bug whereby the audio files weren't generating in all Internet Explorer setups. Fixed a bug with the "don't force registered users to enter the anti-spam word" setting. Changed table creation method so that PHP installations with low memory allocated to scripts will still be able to create the necessary tables. Also, made all display strings more easily customizable, to enhance translation support.
2007-09-03  Version 2.9.1  Tweaked the diagnosis page to reflect user fixes. Also, added more accurate error catching for when the anti-spam word doesn't exist in the database.
2007-08-11  Version 2.9.0  New option to generate audio files for visually-impaired users. It now also makes use of a database to prevent mapping of words.

View all previous change items at http://www.theblog.ca/anti-spam-history
*/

// -----------------------------------------------------------------------------------
// Customize these parts if you want to change the defaults.

$cas_text = array(); // <-- This line should not be changed
// List as many words as you like, one per line
// If you want some words to be used more often, enter them multiple times.
// It is best to use words that are eight letters or less
$cas_text[] = 'fine';
$cas_text[] = 'space';
$cas_text[] = 'pick';
$cas_text[] = 'near';
$cas_text[] = 'cell';
$cas_text[] = 'sea';
$cas_text[] = 'trout';
$cas_text[] = 'gone';
$cas_text[] = 'borrow';
$cas_text[] = 'better';

// Set this to equal TRUE if you want to force registered users to enter the anti-spam word as well.
$cas_forcereg = false;

// Set this to equal TRUE if you want to allow trackbacks (but be vulnerable to trackback spam)
$cas_allowtrack = true;

// Set this to equal TRUE if you want to send all trackbacks to the moderation queue (only works if the above setting is TRUE)
$cas_modtrack = false;

// Set this to equal TRUE if you want to allow pingbacks (but be vulnerable to pingback spam)
$cas_allowping = true;

// Set this to equal TRUE if you want to send all pingbacks to the moderation queue (only works if the above setting is TRUE)
$cas_modping = false;

$cas_fontlist = array(); // <-- This line should not be changed
// List as many TrueType font(s) as you like, one per line. Drop your own font files into this plugin's directory.
// If you are using your own fonts, make sure all fonts used are about the same default size.
// If you want some fonts to be used more frequently, enter them multiple times.
// Default freeware fonts from fonts101.com
$cas_fontlist[] = "04b03.ttf";
$cas_fontlist[] = "baileysansitcbook.ttf";
$cas_fontlist[] = "decorative-stylisticblackout-regular.ttf";
$cas_fontlist[] = "okrienhmk.ttf";
$cas_fontlist[] = "pointed.ttf";
$cas_fontlist[] = "typoderm.ttf";

// Set the anti-spam image width and height.
// You may need to increase these sizes for longer words and/or bigger fonts.
$cas_imgwidth = 160;
$cas_imgheight = 50;

// Set this to TRUE if you want to use random text colors.
// If random colors are not selected, blue text will appear on a white background, and white text will appear on black background (as decided in the next option)
$cas_randomcolors = true;

// Set the background color for the anti-spam image.
// Choose either "black" or "white"
$cas_bgcolorset = "white";

// Set the border color for the anti-spam image.
// Write either major colors (red, green, blue, etc.) or enter the HTML color code (such as #C0C0C0)
$cas_borderclr = "black";

// Set this to TRUE if you prefer PNG graphics (better quality text)
// Set this to FALSE if you prefer more compatable graphics (PNG crashes IE 4; JPEG does not)
$cas_UsePngNotJpeg = false;

// Set this to TRUE if you will be editing your comments file (add this php line wherever you want the anti-spam image inserted in the comments.php file: do_action('secure_image', $post->ID); )
// Set this to FALSE if you want to use the default Javascript positioning
$cas_manualinsert = false;

// Leave this as FALSE unless you are instructed by an error message to "The site administrator needs to manually configure his/her site address in the plugin configuration file!" In that case, enter your site address, which should be the same as the WordPress address from your Options page. Enter it in quotes like the commented line below, withOUT a trailing slash.
// $cas_siteurl = "http://yoursiteaddress/whatever";
$cas_siteurl = false;

// Set this to TRUE to enable wav files of the anti-spam words so that visually enabled users can comment
$cas_wav = true;

// Set this to FALSE if you know that your server doesn't have the sox filter or if you do not want to stretch the audio files by a variable rate
$cas_sox = true;

// Set this to TRUE to insert anti-spam protection to the registration form
$cas_reg_protection = false;

$cas_reg_blacklist = array();
// Registration blacklist. Enter either e-mail address or domains (like 'theblog.ca') of e-mail addresses you don't want to allow
// Be careful, because if you enter 'theblog.ca' if an e-mail address contains that string anywhere, it will be blocked
$cas_reg_blacklist[] = 'exampleaddressthatwontmatch.bc.ca';


// -----------------------------------------------------------------------------------
// Translation stuff. If your blog is in a different language, edit these fields to suit your language!
$cas_displaytext = array(); // <-- This line should not be changed

// This is the name of the label for the comment form
$cas_displaytext['label'] = 'Anti-spam word: (Required)';

// These are the instructions for filling in the anti-spam word in the comment form
$cas_displaytext['instructions'] = 'To prove you\'re a person (not a spam script), type the security word shown in the picture.';

if ($cas_wav) {
// Additional instructions for the audio-impaired users
$cas_displaytext['instructions'] .= ' Click on the picture to hear an audio file of the word.';
}

// Error message if someone has not typed anything into the anti-spam field
$cas_displaytext['emptyfield'] = 'Error: Please enter the anti-spam word.';

// Error message if the particular anti-spam image has already been used on a comment
$cas_displaytext['alreadyused'] = 'Error: The anti-spam word is invalid. Please report this error to the webmaster. Go back and refresh the page to re-submit your comment.';

// Error message if someone has typed the wrong word into the anti-spam field
$cas_displaytext['wrongfield'] = 'Error: Please enter the correct anti-spam word. Press the back button and try again.';

// Error message instructions to copy the text of the comment before pressing the back button:
$cas_displaytext['copyfield'] = 'Copy your comment in case this site forces a page reload whenever you press the Back button:';

// Error message when trying to generate an audio file and the anti-spam image has already been used
$cas_displaytext['not_valid'] = 'That anti-spam number is no longer valid.';

// Text to show in an invalid image
$cas_displaytext['invalid'] = '* * * INVALID * * *';

// Error message to point the webmaster to edit the plugin configuration settings
$cas_displaytext['manually_configure'] = 'The site administrator needs to manually configure his site address in the plugin configuration file!';

// Error message if the GD Library is not installed
$cas_displaytext['no_gd'] = 'Cannot initialize new GD image stream';

if (!$cas_wav) {
// Text for the normal alt tag of the image
$cas_displaytext['alt_tag'] = 'Anti-spam image';
}
else {
// Text for the alt tag of the image for visually impaired users
$cas_displaytext['alt_tag'] = 'Click to hear an audio file of the anti-spam word';
}

// Error messages for the registration form only 

// Error if the particular anti-spam image has already been used
$cas_displaytext['register_alreadyused'] = 'ERROR: The anti-spam word is no longer valid.';

// Error message if someone has typed the wrong answer into the anti-spam field
$cas_displaytext['register_wrongfield'] = 'ERROR: Please enter the correct anti-spam word.';

// Error message if someone has typed a blocked e-mail address
$cas_displaytext['register_blocked'] = 'ERROR: That e-mail address has been blocked.';

// -----------------------------------------------------------------------------------
// You should not need to (and probably shouldn't) edit anything from here to the end.

// This makes it so that the invalid audio file pages are not indexed by search engines
$cas_displaytext['not_valid'] = '<html><head><title>' . $cas_displaytext['not_valid'] . '</title><meta name="Robots" content="noindex,nofollow"></head><body>' . $cas_displaytext['not_valid'] . '</body></html>';

// Define the path to the plugin
$cas_folder = str_replace('\\', '/', dirname(__FILE__));
$cas_thisfolder = explode('/', $cas_folder);
$cas_fontpath =  $cas_folder. '/';

global $wpdb; // Must be global for WordPress 2.5
global $cas_table; // Must be global for WordPress 2.5
// Name of the table that holds all the anti-spam words
$cas_table = $wpdb->prefix . "cas_image";

global $cas_count; // Must be global for WordPress 2.5
// Name of the table that holds the count for the anti-spam words
$cas_count = $wpdb->prefix . "cas_count";

// Determine how many words were entered
$cas_textcount = count( $cas_text );
// Copy the first element to a new last element
$cas_text[] = $cas_text[0];
// Set the first element to invalid
$cas_text[0] = "* * * INVALID * * *";

// Output the antispam audio file
if( isset( $_GET['audioselect'] ) )
{
	// Include the wav merging class
	include $cas_fontpath . 'wav_join.php';

	// Validate the input values
	$cas_audioselect = intval( $_GET['audioselect'] );

	// Get the word from the database
	$cas_antispam = $wpdb->get_var('SELECT word FROM ' . $cas_table . ' WHERE id = ' . $cas_audioselect);
	if (is_null($cas_antispam)) die($cas_displaytext['not_valid']);

	// Get the word
	$petersword = strtolower($cas_antispam);

	// Strip out spaces and hyphens
	$petersword = str_replace(array('-',' '), array('',''), $petersword);

	// Count the number of letters in the word
	$word_count = strlen($petersword);

	// Write the function str_split if it doesn't exist (in PHP 4)
	if (!function_exists('str_split')) {
		function str_split($string) {
			global $word_count;
			$letters = array();
			for ($i = 0; $i < $word_count; ++$i) {
				$letters[$i] = substr($string, $i, 1);
			}
			return $letters;
		}
	}

	// Split the word into an array of individual letters
	$cas_wavs = str_split($petersword);

	// Merge the wav files for this word
	$cas_one_wav = joinwavs($cas_wavs);

	// Create a temporary file for this word
	$cas_audioinput = tempnam($cas_fontpath, 'cas_');
	$cas_audiooutput = $cas_audioinput . '1.wav';
	$cas_handle=fopen($cas_audioinput, "w");
	fwrite($cas_handle, $cas_one_wav);
	fclose($cas_handle);

	// Clear the variable out since the sound is now in a temporary file
	unset($cas_one_wav);

    if ($cas_sox) {
        // Make use of the sox filter, if available, to make it hard for bots to crack the audio file

        // This will determine a stretch factor with 20 different possibilities
        $stretch_random = sprintf("%02d", rand( 0, 20));

        // Stretch the audio file
        @exec('sox ' . $cas_audioinput . ' ' . $cas_audiooutput . ' stretch 1.' . $stretch_random . ' > /dev/null 2>&1');
    }
    
	// If the sox filter was available, use the stretched file. Otherwise, use the original file
	if (file_exists($cas_audiooutput)) {
		$cas_one_wav = file_get_contents($cas_audiooutput);
	}
	else {
		$cas_one_wav = file_get_contents($cas_audioinput);
	}

	// Spit it the file out to the browser	
	output('output.wav', $cas_one_wav);

	// Delete the files
	if (file_exists($cas_audioinput)) unlink($cas_audioinput);
	if (file_exists($cas_audiooutput)) unlink($cas_audiooutput);

	// Prevent any more display to the page
	die();
}

// Output the antispam image
elseif( isset( $_GET['antiselect'] ) )
{
	// Pick a random font to use
	$cas_font = $cas_fontpath . $cas_fontlist[ rand( 0, count( $cas_fontlist ) - 1 ) ];

	// Set the default colors for when random text colors are not selected
	if ( $cas_bgcolorset == "white") {
	$cas_textcolor = array( 0, 0, 255 ); // blue text
	$cas_bgcolor = array( 255, 255, 255); // white background
	}
	else {
	$cas_textcolor = array( 255, 255, 255 ); // white text
	$cas_bgcolor = array( 0, 0, 0); // black background
	}

	// If selected, pick a random color for the antispam word text
	if( $cas_randomcolors )
	{
		$cas_rand = rand( 0, 4 );
		switch( $cas_bgcolorset )
			{
				case "white":
					$cas_textcolorchoice[0] = array ( 0, 0, 255 ); // blue
					$cas_textcolorchoice[1] = array ( 0, 153, 0 ); // greenish
					$cas_textcolorchoice[2] = array ( 204, 0, 0 ); // reddish
					$cas_textcolorchoice[3] = array ( 203, 0, 154 ); // purplish
					$cas_textcolorchoice[4] = array ( 0, 0, 0 ); // black
					$cas_textcolor = $cas_textcolorchoice[$cas_rand];
					break;
				default:
					$cas_textcolorchoice[0] = array ( 255, 255, 0 ); // yellow
					$cas_textcolorchoice[1] = array ( 0, 255, 255 ); // blueish
					$cas_textcolorchoice[2] = array ( 255, 153, 204 ); // pinkish
					$cas_textcolorchoice[3] = array ( 102, 255, 102 ); // greenish
					$cas_textcolorchoice[4] = array ( 255, 255, 255 ); // white
					$cas_textcolor = $cas_textcolorchoice[$cas_rand];
					break;
			}
	}

	// Validate the input values
	$cas_antiselect = intval( $_GET['antiselect'] );

	// Get the word from the database
	$cas_antispam = $wpdb->get_var('SELECT word FROM ' . $cas_table . ' WHERE id = ' . $cas_antiselect);
	if (is_null($cas_antispam)) $cas_antispam = $cas_displaytext['invalid'];

	// Start building the image
	$cas_image = @imagecreate( $cas_imgwidth, $cas_imgheight ) or die($cas_displaytext['no_gd']);
	$cas_bgcolor = imagecolorallocate( $cas_image, $cas_bgcolor[0], $cas_bgcolor[1], $cas_bgcolor[2] );
	$cas_fontcolor = imagecolorallocate( $cas_image, $cas_textcolor[0], $cas_textcolor[1], $cas_textcolor[2] );

	// Check for freetype lib, if not found default to ugly built in capability using imagechar (Lee's mod)
	// Also check that the chosen TrueType font is available
	if( function_exists( 'imagettftext' ) && file_exists( $cas_font ) )
	{
		$cas_angle = 4; // Degrees to tilt the text
		$cas_offset = 10; // Pixels to offset the text from the border
		$cas_fontsize = 28; // Default font size for the anti-spam image
		$cas_imagebox = imagettfbbox($cas_fontsize, $cas_angle, $cas_font, $cas_antispam);
		$cas_boxwidth = $cas_imagebox[2] - $cas_imagebox[0];
		$cas_boxheight = $cas_imagebox[1] - $cas_imagebox[7];
		
		// if the text width is too big for the image, decrease the font size to a certain extent (best practice is of course not to use really long words!
		while ($cas_boxwidth > $cas_imgwidth - $cas_offset && $cas_fontsize > 12) {
			$cas_fontsize = $cas_fontsize - 3;
			$cas_imagebox = imagettfbbox($cas_fontsize, $cas_angle, $cas_font, $cas_antispam);
			$cas_boxwidth = $cas_imagebox[2] - $cas_imagebox[0];
		}

		// if the text height too big for the image, decrease the font size to a certain extent (best practice is of course not to use really long words!
		while ($cas_boxheight > $cas_imgheight - $cas_offset && $cas_fontsize > 12) {
			$cas_fontsize = $cas_fontsize - 3;
			$cas_imagebox = imagettfbbox($cas_fontsize, $cas_angle, $cas_font, $cas_antispam);
			$cas_boxheight = $cas_imagebox[1] - $cas_imagebox[7];
		}

		// Use png is available, since it produces clearer text images
		if( function_exists( 'imagepng' ) && $cas_UsePngNotJpeg )
		{
			imagettftext( $cas_image, $cas_fontsize, $cas_angle, $cas_offset, $cas_imgheight - $cas_offset, $cas_fontcolor, $cas_font, $cas_antispam );
			header( "Content-type: image/png" );
			imagepng( $cas_image );
		} else {
			imagettftext( $cas_image, $cas_fontsize, $cas_angle, $cas_offset, $cas_imgheight - $cas_offset, $cas_fontcolor, $cas_font, $cas_antispam );
			header( "Content-type: image/jpeg" );
			imagejpeg( $cas_image );
		}
	} else {
		$cas_fontsize = 5; // 1, 2, 3, 4 or 5 (higher numbers correspond to larger font sizes)
		$tmp_len = strlen( $cas_antispam );
		for( $tmp_count = 0; $tmp_count < $tmp_len; $tmp_count++ )
		{
		   $tmp_xpos = $tmp_count * imagefontwidth( $cas_fontsize ) + 20;
		   $tmp_ypos = 10;
		   imagechar( $cas_image, $cas_fontsize, $tmp_xpos, $tmp_ypos, $cas_antispam, $cas_fontcolor );
		   $cas_antispam = substr( $cas_antispam, 1);   
		}
		header("Content-Type: image/gif");
		imagegif( $cas_image );
	} // end if
	imagedestroy( $cas_image );
	die();
} else {

	// Determine the blog url to this script
	$cas_message = "<small>{$cas_displaytext['instructions']}</small>";
	if ( function_exists(get_option) && !$cas_siteurl) {
		$cas_siteurl = get_option('siteurl');
		}
	if ( !$cas_siteurl ) {
		$cas_message = "<h1>{$cas_displaytext['manually_configure']}</h1>";
		}
	$cas_myurl = $cas_siteurl;

	class PeterAntiSpam
	{
		function PeterAntiSpam()
		{
			global $cas_manualinsert;
			add_action( 'secure_image', array( &$this, 'comment_form' ) );    // add image and input field to comment form
			if (! $cas_manualinsert ) {
				add_action( 'comment_form', array( &$this, 'comment_form' ) );    // add image and input field to comment form
			}
			add_filter( 'preprocess_comment', array( &$this, 'comment_post') );    // add post comment post security code check
		}

		function comment_form()
		{
			global $cas_forcereg, $user_ID, $cas_textcount, $cas_text, $cas_myurl, $cas_imgheight, $cas_imgwidth, $cas_limitcolor, $cas_manualinsert, $cas_borderclr, $cas_message, $cas_displaytext, $cas_table, $cas_count, $cas_wav, $wpdb;
			// If the user is logged in, don't prompt for code
			if( ! $cas_forcereg && intval( $user_ID ) > 0 )
			{
				return( $post_ID );
			}

			// Insert a row into the count database to generate an auto_increment number
			$wpdb->query('INSERT INTO ' . $cas_count . ' (id) VALUES (NULL)');

			// Get the id of the inserted count to feed to the word table and image generator
			$cas_rowid = $wpdb->get_var('SELECT last_insert_id()');

			// Pick a random number
			$cas_antiselect = rand( 1, $cas_textcount ); // 0 is for invalid, so don't select it

			// Put the word corresponding to the random number into the database
			$wpdb->query('INSERT INTO ' . $cas_table . ' (id, createtime, word) VALUES (' . $cas_rowid . ', ' . time() . ', \'' . $cas_text[$cas_antiselect] . '\')');

			// Delete the row from the count table
			$wpdb->query('DELETE FROM ' . $cas_count . ' WHERE id = ' . $cas_rowid);

			// Do some table admin while we can :D
			if (strlen($cas_rowid) == 10) {

				// Delete all rows from the count table
				$wpdb->query('DELETE FROM ' . $cas_count);

				// Reset the table's auto increment if it's getting too huge
				$wpdb->query('ALTER TABLE ' . $cas_count . ' AUTO_INCREMENT=1');

				// Delete any anti-spam words more than a day old
				$wpdb->query('DELETE FROM ' . $cas_table . ' WHERE ' . time() . ' > createtime + 86400');
			}

			echo( "\t\t\t".'<div style="display:block;" id="secureimgdiv">'."\n\t\t\t\t" );
			echo( "<p><label for=\"securitycode\">{$cas_displaytext['label']}</label><span style=\"color:#FF0000;\">*</span><br />\n\t\t\t\t" );
			echo( $cas_message . "<br />\n\t\t\t\t" );
			echo( '<input type="text" name="securitycode" id="securitycode" size="30" tabindex="4" />'."\n\t\t\t\t" );
			echo( '<input type="hidden" name="matchthis" value="' . $cas_rowid . "\" />\n\t\t\t\t" );
			if ($cas_wav) echo( '<a href="' . $cas_myurl . '?audioselect=' . $cas_rowid . '" rel="nofollow" title="' . $cas_displaytext['alt_tag'] . '">' . "\n\t\t\t\t" );
			echo( '<img src="' . $cas_myurl . '?antiselect=' . $cas_rowid . '" ' );
			echo( 'alt="' . $cas_displaytext['alt_tag'] . '" ' );
			echo( 'style="border:1px solid ' . $cas_borderclr . ';vertical-align:top;' );
			echo( 'height:' . $cas_imgheight .'px;width:' . $cas_imgwidth . 'px;" />' );
			if ($cas_wav) echo( '</a>');
			echo( "</p>\n\t\t\t" );
			echo( "</div>\n\t\t\t" );
if ( ! $cas_manualinsert ) {
			echo"<script language='JavaScript' type='text/javascript'>
			<!--
				var urlinput = document.getElementById(\"url\");
				var submitp = urlinput.parentNode;
				var substitution2 = document.getElementById(\"secureimgdiv\");
				submitp.appendChild(substitution2, urlinput);
			// -->
			</script>\n";
}
			return( $post_ID );
		}

		function comment_post( $incoming_comment )
		{
			global $_POST, $cas_text, $cas_textcount, $cas_forcereg, $user_ID, $cas_allowtrack, $cas_allowping, $cas_displaytext, $cas_modping, $cas_modtrack, $cas_table, $wpdb;
			// Validate the form input values
			if( isset( $_POST['securitycode'] ) )
			{
				// Consider only the first 50 characters in the posted word
				$securitycode = substr( strval( $_POST['securitycode'] ), 0, 50 );

				// Remove all spaces and hyphens to give the commenter a break!
				$securitycode = str_replace(' ', '', $securitycode);
				$securitycode = str_replace('-', '', $securitycode);

			} else {
				$securitycode = '';
			}
			if( isset( $_POST['matchthis'] ) )
			{
				$matchnum = intval( $_POST['matchthis'] );
			} else {
				$matchnum = 0;
			}

			// If the user is not logged in check the security code
			if( $cas_forcereg || ! isset( $user_ID ) )
			{
				$istrackping = $incoming_comment['comment_type'];
				$commentbody = $incoming_comment['comment_content'];
				if ( $istrackping == 'pingback' && $cas_allowping ) {

					// Send all pingbacks to a moderation queue?
					if ($cas_modping) add_filter('pre_comment_approved', create_function('$mod_ping', 'return \'0\';'));
				}
				elseif ( $istrackping == 'trackback' && $cas_allowtrack ) {

					// Send all trackbacks to a moderation queue?
					if ($cas_modtrack) add_filter('pre_comment_approved', create_function('$mod_track', 'return \'0\';'));
				}
				else
				{
					if ( $securitycode == '' )
					{
						die("<p>{$cas_displaytext['emptyfield']}</p>\n<p>{$cas_displaytext['copyfield']}</p>\n<textarea cols=\"100%\" rows=\"10\" onclick=\"this.select();\" readonly=\"true\">$commentbody</textarea>");
					}

					// Get the anti-spam word from the database
					$matchthis = $wpdb->get_var('SELECT word FROM ' . $cas_table . ' WHERE id = ' . $matchnum);

					// If this row doesn't exist, say something
					if (is_null($matchthis)) die("<p>{$cas_displaytext['alreadyused']}</p>\n<p>{$cas_displaytext['copyfield']}</p>\n<textarea cols=\"100%\" rows=\"10\" onclick=\"this.select();\" readonly=\"true\">$commentbody</textarea>");

					// Remove all spaces and hyphens, since we removed them from what the commenter entered
					$matchthis = str_replace(' ', '', $matchthis);
					$matchthis = str_replace('-', '', $matchthis);

					if ( strtolower( $matchthis ) != strtolower( $securitycode ) )
					{
						die("<p>{$cas_displaytext['wrongfield']}</p>\n<p>{$cas_displaytext['copyfield']}</p>\n<textarea cols=\"100%\" rows=\"10\" onclick=\"this.select();\" readonly=\"true\">$commentbody</textarea>");
					} else {
						// The word matched, so delete the row for the anti-spam word so that it cannot be used again
						$wpdb->query('DELETE FROM ' . $cas_table . ' WHERE id = ' . $matchnum);
						unset( $matchthis );

						// Do some more table admin while we can :D
						// Delete any anti-spam words more than a day old
						$wpdb->query('DELETE FROM ' . $cas_table . ' WHERE ' . time() . ' > createtime + 86400');
					}
				}
			}
			return( $incoming_comment );
		}
	}
	new PeterAntiSpam();

// Add some troubleshooting to the Manage tab

function cas_manage() {
global $cas_siteurl;
print "<div class=\"wrap\">\n";
print "<h2>Peter's Custom Anti-Spam</h2>\n";
print "<p>This page will diagnose your server setup to see how it relates to the performance of this plugin. Remember that all actual settings for this plugin can be changed under the Plugins &gt; Plugin Editor menu.</p>\n";
print "<hr />\n";
print "<p>Can't see the anti-spam image when you're viewing a comment form? Remember to log out of WordPress first -- by default, the image doesn't show to registered users (although you can edit that option in the plugin file).</p>\n";
print "<hr />\n";

print "<p><strong>GD library</strong><br />\n";
if (function_exists(imagejpeg)) {
print "Yay! The GD library is installed. This is the most important thing to be able to use this plugin. You might still need to do some tweaking to the settings of course.</p>\n";
}
else {
print "<font color=\"red\">The GD library is not installed. This plugin will not work without it. Ask your webhost to install this library (or install it if you manage your own server).</font></p>\n";
}

print "<p><strong>FreeType</strong><br />\n";
if (function_exists(imagettftext)) {
print "Yay! The FreeType library is installed. The anti-spam image should display using the uploaded fonts.</p>\n";
}
else {
print "<font color=\"red\">The FreeType library is not installed. The anti-spam image should still display, but with only a plain font. Ask your webhost to install this library (or install it if you manage your own server) if you want to be able to use the uploaded fonts.</font></p>\n";
}

print "<p><strong>get_option</strong><br />\n";
if (function_exists(get_option)) {
print "Yay! The script should be able to automatically figure out your blog address.</p>\n";
}
elseif ($cas_siteurl) {
print "The script cannot automatically figure out your blog address, but you have entered it manually. This should be the correct address to your blog: $cas_siteurl. Otherwise, please make sure you correct the \$cas_siteurl value in the plugin file (something like <em>http://www.yoursiteaddress.com/whatever</em>).";
}
else {
print "<font color=\"red\">For some reason, the script cannot access your blog address. Make sure you edit the \$cas_siteurl value in the plugin file (something like <em>http://www.yoursiteaddress.com/whatever</em>).</font></p>\n";
}

print "<hr />\n";
print "<p>Need more help? Post a comment on the <a href=\"http://www.theblog.ca/?p=21\" title=\"Where all information is posted about the plugin\">Peter's Custom Anti-Spam page</a>.</p>\n";
print "</div>\n";
}

function cas_adminmenu() {
	add_management_page('Custom anti-spam', 'Custom anti-spam', 7, 'custom_anti_spam.php', 'cas_manage');
}

add_action('admin_menu','cas_adminmenu',1);

// Install the anti-spam word database table upon plugin activation

function cas_install () {
	global $wpdb, $cas_table, $cas_count;

	// Add the table to hold the anti-spam words
	if($wpdb->get_var("SHOW TABLES LIKE '$cas_table'") != $cas_table) {
		$sql = "CREATE TABLE " . $cas_table . " (
	  	id int(10) NOT NULL,
	  	createtime int(10) NOT NULL,
	  	word VARCHAR(20) NOT NULL,
	  	UNIQUE KEY id (id)
		);";
      	$wpdb->query($sql);
	}

	// Add the table to hold the count for the anti-spam words
	if($wpdb->get_var("SHOW TABLES LIKE '$cas_count'") != $cas_count) {
		$sql = "CREATE TABLE " . $cas_count . " (
	  	id int(10) NOT NULL AUTO_INCREMENT,
	  	UNIQUE KEY id (id)
		);";
      	$wpdb->query($sql);
	}
      	
}

// Remove the anti-spam word database table upon plugin de-activation

function cas_uninstall () {
	global $wpdb, $cas_table, $cas_count;

	if($wpdb->get_var("SHOW TABLES LIKE '$cas_table'") == $cas_table) {
		$sql = "DROP TABLE " . $cas_table;
		$wpdb->query($sql);
	}

	if($wpdb->get_var("SHOW TABLES LIKE '$cas_count'") == $cas_count) {
		$sql = "DROP TABLE " . $cas_count;
		$wpdb->query($sql);
	}
}

add_action('activate_' . end($cas_thisfolder) . '/custom_anti_spam.php', 'cas_install');
add_action('deactivate_' . end($cas_thisfolder) . '/custom_anti_spam.php', 'cas_uninstall');

function cas_register_form() {
	global $cas_text, $cas_textcount, $cas_myurl, $cas_imgheight, $cas_imgwidth, $cas_limitcolor, $cas_borderclr, $cas_message, $cas_displaytext, $cas_table, $cas_count, $cas_wav, $wpdb;

	// Insert a row into the count database to generate an auto_increment number
	$wpdb->query('INSERT INTO ' . $cas_count . ' (id) VALUES (NULL)');

	// Get the id of the inserted count to feed to the word table and image generator
	$cas_rowid = $wpdb->get_var('SELECT last_insert_id()');

	// Pick a random number
	$cas_antiselect = rand( 1, $cas_textcount ); // 0 is for invalid, so don't select it

	// Put the word corresponding to the random number into the database
	$wpdb->query('INSERT INTO ' . $cas_table . ' (id, createtime, word) VALUES (' . $cas_rowid . ', ' . time() . ', \'' . $cas_text[$cas_antiselect] . '\')');
	
	// Delete the row from the count table
	$wpdb->query('DELETE FROM ' . $cas_count . ' WHERE id = ' . $cas_rowid);

	// Do some table admin while we can :D
	if (strlen($cas_rowid) == 10) {

		// Delete all rows from the count table
		$wpdb->query('DELETE FROM ' . $cas_count);

		// Reset the table's auto increment if it's getting too huge
		$wpdb->query('ALTER TABLE ' . $cas_count . ' AUTO_INCREMENT=1');

		// Delete any anti-spam words more than a day old
		$wpdb->query('DELETE FROM ' . $cas_table . ' WHERE ' . time() . ' > createtime + 86400');
	}

	echo( "\t\t\t".'<div style="display:block;" id="secureimgdiv">'."\n\t\t\t\t" );
	echo( "<p><label for=\"securitycode\">{$cas_displaytext['label']}</label><span style=\"color:#FF0000;\">*</span><br />\n\t\t\t\t" );
	echo( $cas_message . "<br />\n\t\t\t\t" );
	echo( '<input type="text" name="securitycode" id="securitycode" size="30" tabindex="4" />'."\n\t\t\t\t" );
	echo( '<input type="hidden" name="matchthis" value="' . $cas_rowid . "\" />\n\t\t\t\t" );
	if ($cas_wav) echo( '<a href="' . $cas_myurl . '?audioselect=' . $cas_rowid . '" title="' . $cas_displaytext['alt_tag'] . '">' );
		echo( '<img src="' . $cas_myurl . '?antiselect=' . $cas_rowid . "\"\n\t\t\t\t" );
		echo( 'alt="' . $cas_displaytext['alt_tag'] . '" ' );
		echo( 'style="border:1px solid ' . $cas_borderclr . ';vertical-align:top;' );
		echo( 'height:' . $cas_imgheight .'px;width:' . $cas_imgwidth . 'px;" />' );
		if ($cas_wav) echo( '</a>');
		echo( "</p>\n\t\t\t" );
		echo( "</div>\n\t\t\t" );
}

function cas_register_post() {
	global $errors, $_POST, $cas_text, $cas_textcount, $user_ID, $cas_displaytext, $cas_table, $wpdb, $cas_reg_blacklist;

		// Validate the form input values
		if( isset( $_POST['securitycode'] ) )
		{
			// Consider only the first 50 characters in the posted word
			$securitycode = substr( strval( $_POST['securitycode'] ), 0, 50 );

			// Remove all spaces and hyphens to give the commenter a break!
			$securitycode = str_replace(' ', '', $securitycode);
			$securitycode = str_replace('-', '', $securitycode);

		} else {
			$securitycode = '';
		}
		if( isset( $_POST['matchthis'] ) )
		{
			$matchnum = intval( $_POST['matchthis'] );
		} else {
			$matchnum = 0;
		}

		if ( $securitycode == '' )
		{			
			$errors['cas_register'] = $cas_displaytext['emptyfield'];
		}
		else {
			// Get the anti-spam word from the database
			$matchthis = $wpdb->get_var('SELECT word FROM ' . $cas_table . ' WHERE id = ' . $matchnum);

			// If this row doesn't exist, say something
			if (is_null($matchthis)) {
				$errors['cas_register'] = $cas_displaytext['register_alreadyused'];
			}

			else {

				// Remove all spaces and hyphens, since we removed them from what the commenter entered
				$matchthis = str_replace(' ', '', $matchthis);
				$matchthis = str_replace('-', '', $matchthis);

				// Check what was entered against what the code should be
				if ( strtolower( $matchthis ) != strtolower( $securitycode ) ) {
					$errors['cas_register'] = $cas_displaytext['register_wrongfield'];
				}

				else {
					// The word matched, so delete the row for the anti-spam word so that it cannot be used again
					$wpdb->query('DELETE FROM ' . $cas_table . ' WHERE id = ' . $matchnum);
					unset( $matchthis );

					// Do some more table admin while we can :D
					// Delete any anti-spam words more than a day old
					$wpdb->query('DELETE FROM ' . $cas_table . ' WHERE ' . time() . ' > createtime + 86400');
				}
			}
		}
		foreach ($cas_reg_blacklist as $cas_blacklist) {
			if (stristr($_POST['user_email'], $cas_blacklist)) {
				$errors['cas_register'] = $cas_displaytext['register_blocked'];
				break;
			}
		}
	}

// Add registration protection to the appropriate hooks only if it has been enabled in this plugin's settings
if ($cas_reg_protection) {
	add_action('register_form', 'cas_register_form');
	add_action('register_post', 'cas_register_post');
	}
}
?>